<?php
/*
Plugin Name: bbPress Code Snippets
Description: Automatically display HTML/PHP code posted in bbPress topics and replies.
Version: 1.0.3
Author: Jason Bobich
Author URI: http://jasonbobich.com
License: GPL2

    Copyright 2012  Jason Bobich

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License version 2,
    as published by the Free Software Foundation.

    You may NOT assume that you can use any other version of the GPL.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    The license for this software can likely be found here:
    http://www.gnu.org/licenses/gpl-2.0.html

*/

/**
 * Initiate plugin
 *
 * @since 1.0.0
 */
function themeblvd_bb_code_snippets(){
	
	// Allow normal user's to use <pre> tags.
	add_action( 'init', 'themeblvd_bb_add_pre_tags' );

	// Convert HTML entities between <pre> and <code> tags
	// before saving to the DB so WP doesn't filter them 
	// out with wp_filter_kses
	add_filter( 'bbp_new_reply_pre_content',  'themeblvd_bb_save_code', 9 );
	add_filter( 'bbp_new_topic_pre_content',  'themeblvd_bb_save_code', 9 );
	add_filter( 'bbp_edit_reply_pre_content', 'themeblvd_bb_save_code', 9 );
	add_filter( 'bbp_edit_topic_pre_content', 'themeblvd_bb_save_code', 9 );

	// Display HTML entities back to the user when editing their 
	// topic or reply.
	add_filter( 'bbp_get_form_reply_content', 'htmlspecialchars_decode' );
	add_filter( 'bbp_get_form_topic_content', 'htmlspecialchars_decode' );
	
	// Remove bbPress wpautop on replies/topics
	remove_filter( 'bbp_get_reply_content', 'make_clickable', 9 );
	remove_filter( 'bbp_get_topic_content', 'make_clickable', 9 );
	remove_filter( 'bbp_get_reply_content', 'wpautop', 30 );
	remove_filter( 'bbp_get_topic_content', 'wpautop', 30 );
	
	// And now, loop through and put wpautop back on the chunks that 
	// are not in <pre> tags.
	add_filter( 'bbp_get_reply_content', 'themeblvd_bb_content_formatter' );
	add_filter( 'bbp_get_topic_content', 'themeblvd_bb_content_formatter' );
	
}
add_action( 'after_setup_theme', 'themeblvd_bb_code_snippets' );

/**
 * Before saving topics and replies, convert HTML entities so 
 * they'll save in a way we can display them between 
 * <pre> and <code>.
 * 
 * @since 1.0.0
 *
 * @param string $content Content retrieved for reply or topic
 * @return string $content Content after HTML etities have been converted
 */
function themeblvd_bb_save_code( $content ){
	// Format HTML entities
	$content = preg_replace_callback('#<(code|pre)([^>]*)>(((?!</?\1).)*|(?R))*</\1>#si', 'themeblvd_bb_convert_code', $content );
	return $content;
}

/**
 * Callback for preg_replace_callback to 
 * convert HTML entities.
 *
 * @since 1.0.0
 *
 * @param array $matches Matches from preg_replace_callback
 * @return string $code_block Code block after HTML entities have been converted
 */
function themeblvd_bb_convert_code( $matches ) {  
    // Create code block
    $code_block = '<'.$matches[1].$matches[2].'>'.htmlspecialchars(substr(str_replace('<'.$matches[1].$matches[2].'>', '', $matches[0]), 0, -(strlen($matches[1]) + 3))).'</'.$matches[1].'>';
    return $code_block;
}

/**
 * Format content for replies and topics assuming that 
 * wpautop has been removed.
 * 
 * @since 1.0.0
 *
 * @param string $content Content retrieved for reply or topic
 * @return string $new_content Final content chunks put back together
 */
function themeblvd_bb_content_formatter( $content ) {

	$new_content = '';

	// Patterns for <pre> tag
	$pattern_full = '{(<pre>.*?\</pre>)}is';
	$pattern_contents = '{<pre>(.*?)</pre>}is';
	
	// Split into pieces
	$pieces = preg_split( $pattern_full, $content, -1, PREG_SPLIT_DELIM_CAPTURE );
	
	// Loop through and put wpautop on standard content 
	// and put code back in <pre> tags.
	foreach( $pieces as $piece ) {
		if( preg_match( $pattern_contents, $piece, $matches ) ) {
			
			// Display raw content between <pre> tags
			$new_content .= '<pre>'.trim($matches[1]).'</pre>';

		} else {

			// Filter through this content piece for <code> tags
			$pattern_code_full = '{(<code>.*?\</code>)}is';
			$pattern_code_contents = '{<code>(.*?)</code>}is';
			$code_pieces = preg_split( $pattern_code_full, $piece, -1, PREG_SPLIT_DELIM_CAPTURE );
			
			// Only apply make_clickable() if not in a <code> tag.
			$new_piece = '';
			foreach( $code_pieces as $code_piece ) {
				if( preg_match( $pattern_code_contents, $code_piece, $code_matches ) )
					$new_piece .= '<code>'.trim($code_matches[1]).'</code>';
				else
					$new_piece .= make_clickable( $code_piece );
			}
			
			// And now finish it off by applying wpautop to the whole chunk.
			$new_content .= wpautop( $new_piece );
		}
	}
	
	return $new_content;
}

/** 
 * All <pre> tags in replies/topics
 *
 * @since 1.0.1
 */
function themeblvd_bb_add_pre_tags(){
	global $allowedtags;
	$allowedtags['pre'] = array();
}